@@ -232,6 +232,79 @@ class kcwebsocket:
 # def start():
 #     kwebsocket=kcwebsocket()
 #     kwebsocket.start()
+
+
+class DFAFilter():
+    """DFA算法文字过滤器"""
+    __sensitivelist=[]
+    def __init__(self,sensitivelist):
+        """
+        sensitivelist 敏感字符串列表
+        """
+        self.keyword_chains = {}  # 关键词链表
+        self.delimit = '\x00'  # 限定
+        for keyword in sensitivelist:
+            self.__add(str(keyword).strip())
+    def __add(self, keyword):
+        keyword = keyword.lower()  # 关键词英文变为小写
+        chars = keyword.strip()  # 关键字去除首尾空格和换行
+        if not chars:  # 如果关键词为空直接返回
+            return
+        level = self.keyword_chains
+        # 遍历关键字的每个字
+        for i in range(len(chars)):
+            # 如果这个字已经存在字符链的key中就进入其子字典
+            if chars[i] in level:
+                level = level[chars[i]]
+            else:
+                if not isinstance(level, dict):
+                    break
+                for j in range(i, len(chars)):
+                    level[chars[j]] = {}
+                    last_level, last_char = level, chars[j]
+                    level = level[chars[j]]
+                last_level[last_char] = {self.delimit: 0}
+                break
+        if i == len(chars) - 1:
+            level[self.delimit] = 0
+    def filter(self, message, repl=""):
+        """ 获取过滤后的字符串
+
+        message 待处理的字符串
+
+        repl 替换内容
+        """
+        message = message.lower()
+        ret = []
+        start = 0
+        while start < len(message):
+            level = self.keyword_chains
+            step_ins = 0
+            for char in message[start:]:
+                if char in level:
+                    step_ins += 1
+                    if self.delimit not in level[char]:
+                        level = level[char]
+                    else:
+                        ret.append(repl * step_ins)
+                        start += step_ins - 1
+                        break
+                else:
+                    ret.append(message[start])
+                    break
+            else:
+                ret.append(message[start])
+            start += 1
+        return ''.join(ret)
+    def sensitive(self,message):
+        """ 判断字符串是否包含敏感
+
+        message 待处理的字符串
+        """
+        if self.filter(message) == message:
+            return True
+        else:
+            return False
 redis=kcwredis()
 def send_mail(user,text="邮件内容",theme="邮件主题",recNick="收件人昵称"):
     """发送邮件
@@ -947,7 +1020,7 @@ class create:
                         i=0
                         j+=1
                         arr=arr['data']
-                        r=requests.get(arr['dowurl'])
+                        r=requests.get(arr['dowurl'],verify=False)
                         f = open(self.appname+"/"+self.modular+"/controller/"+plug+".zip", "wb")
                         for chunk in r.iter_content(chunk_size=512):
                             if chunk:
