@@ -2,8 +2,20 @@
 from .pymysql import connect,escape_string
 # import config.conf as config
 import kcweb.config as config
-import time,traceback,decimal,random,copy
+import time,traceback,decimal,random,copy,hashlib
+from kcweb.utill.cache import cache as kcwcache
+mycache=copy.deepcopy(config.cache)
+mycache['type']='Python'
+
 dbconfig=config.database
+def md5(strs):
+    """md5加密"""
+    if not strs:
+        return strs
+    m = hashlib.md5()
+    b = strs.encode(encoding='utf-8')
+    m.update(b)
+    return m.hexdigest()
 class mysql:
     """数据库实例"""
     __config=dbconfig
@@ -304,6 +316,82 @@ class mysql:
         self.__table=table
         return self
     __patternerrorcount=0 # 长连接情况下 错误次数
+    def __setcursor(self,typess='DQL'):
+        """设置游标
+
+        参数 type ：数据查询语言DQL，数据操纵语言DML，数据定义语言DDL，数据控制语言DCL
+        """
+        self.__setdbcount()
+        # self.__DBsingle=self.__config['pattern']
+        self.__dbObjcount=self.__config['dbObjcount']
+        self.__errorcount=self.__config['break']
+        self.__connects()
+        bs=''
+        try:
+            if self.__config['deploy']==0: # 集中式(单一服务器)
+                if self.__config['pattern']: # 长连接情况下
+                    bs=self.__masteridentifier
+                    # types='master'
+                    self.__dbobjident=random.randint(0, self.__dbObjcount-1)
+                    self.__cursor=self.__conn[self.__masteridentifier][self.__dbobjident]['obj'].cursor() #获取随机数据库链接实例
+                else:
+                    self.__cursor=self.__conn[self.__masteridentifier].cursor() #获取随机数据库链接实例
+            elif self.__config['deploy']==1: # 分布式(主从服务器)
+                if self.__config['pattern']: # 长连接情况下
+                    bs=self.__masteridentifier
+                    slavecount=self.__dbcount-self.__config['master_num'] #从服务器数量
+                    # types='master'
+                    if typess=='DQL': #数据查询语言DQL
+                        if self.__config['master_dql']:  #主服务器可以执行DQL
+                            if slavecount:
+                                if random.randint(0,1):
+                                    bs=self.__masteridentifier
+                                    # types='master'
+                                    self.__dbobjident=random.randint(0,self.__dbObjcount*self.__config['master_num']-1)
+                                    self.__cursor=self.__conn[self.__masteridentifier][self.__dbobjident]['obj'].cursor() #获取随机数据库链接实例 主服务器
+                                else:
+                                    bs=self.__slaveidentifier
+                                    # types='slave'
+                                    self.__dbobjident=random.randint(0,self.__dbObjcount*slavecount-1)
+                                    self.__cursor=self.__conn[self.__slaveidentifier][self.__dbobjident]['obj'].cursor() #获取随机数据库链接实例 从服务器
+                            else:
+                                bs=self.__masteridentifier
+                                # types='master'
+                                self.__dbobjident=random.randint(0,self.__dbObjcount*self.__config['master_num']-1)
+                                self.__cursor=self.__conn[self.__masteridentifier][self.__dbobjident]['obj'].cursor() #获取随机数据库链接实例 主服务器
+                        else:
+                            bs=self.__slaveidentifier
+                            # types='slave'
+                            self.__dbobjident=random.randint(0,self.__dbObjcount*slavecount-1)
+                            self.__cursor=self.__conn[self.__slaveidentifier][self.__dbobjident]['obj'].cursor() #获取随机数据库链接实例 从服务器
+                    else: #数据操纵语言DML，数据定义语言DDL，数据控制语言DCL
+                        bs=self.__masteridentifier
+                        # types='master'
+                        self.__dbobjident=random.randint(0,self.__dbObjcount*self.__config['master_num']-1)
+                        self.__cursor=self.__conn[self.__masteridentifier][self.__dbobjident]['obj'].cursor() #获取随机数据库链接实例 主服务器
+                    self.__conn[bs][self.__dbobjident]['error']=0
+                else:
+                    self.__cursor=self.__conn[self.__masteridentifier].cursor()
+        except Exception as e: # 铺获未知异常
+            raise Exception(e)
+        else:
+            self.__patternerrorcount=0
+            return self.__cursor
+    
+    def get_exec_cursor(self):
+        """获取执行游标（不推荐使用）
+        
+        参数 type ：数据查询语言DQL，数据操纵语言DML，数据定义语言DDL，数据控制语言DCL
+        """
+        cursor=self.__setcursor("DML")
+        return cursor
+    def get_query_cursor(self):
+        """获取查询游标（不推荐使用）
+        
+        参数 type ：数据查询语言DQL，数据操纵语言DML，数据定义语言DDL，数据控制语言DCL
+        """
+        cursor=self.__setcursor("DQL")
+        return cursor
     def __execute(self,typess='DQL'):
         """执行sql语句
         
@@ -478,8 +566,6 @@ class mysql:
                     i=i+1
                 lists.append(dicts)
             return lists
-    
-
     def select(self,id=None):
         """select查询 
 
@@ -492,12 +578,16 @@ class mysql:
             self.__sqls="("+self.__sql+")"
             self.__None()
             return self.__sqls
-        
+        if self.__cache and self.__cache[0]:
+            cou=kcwcache.cache().set_config(mycache).get_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])))
+            if cou:
+                self.__None()
+                return cou
         self.__execute()
         description=self.__cursor.description #获取字段
         result = self.__cursor.fetchall() #获取查询结果
         self.__cursor.close()
-        self.__None()
+        # self.__None()
         lists=[]
         keys =[]
         for field in description:#获取字段
@@ -511,6 +601,9 @@ class mysql:
                     k=float(row[q])
                 item[keys[q]] = k
             lists.append(item)
+        if lists and self.__cache and self.__cache[0]:
+            kcwcache.cache().set_config(mycache).set_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])),lists,self.__cache[1])
+        self.__None()
         return lists
     def find(self,id=None):
         """查询一条记录
@@ -525,11 +618,16 @@ class mysql:
             self.__sqls="("+self.__sql+")"
             self.__None()
             return self.__sqls
+        if self.__cache and self.__cache[0]:
+            item=kcwcache.cache().set_config(mycache).get_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])))
+            if item:
+                self.__None()
+                return item
         self.__execute()
         description=self.__cursor.description #获取字段
         result = self.__cursor.fetchall() #获取查询结果
         self.__cursor.close()
-        self.__None()
+        # self.__None()
         item = dict()
         keys =[]
         for field in description:#获取字段
@@ -541,8 +639,38 @@ class mysql:
                 if type(row[q])==decimal.Decimal:
                     k=float(row[q])
                 item[keys[q]] = k
+        if item and self.__cache and self.__cache[0]:
+            kcwcache.cache().set_config(mycache).set_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])),item,self.__cache[1])
+        self.__None()
         return item
-
+    def value(self,field):
+        """查询指定字段值
+        
+        返回 一个字段
+        """
+        self.__field=field
+        self.limit(1)
+        self.__setsql()
+        if self.__buildSql:
+            self.__sqls="("+self.__sql+")"
+            self.__None()
+            return self.__sqls
+        if self.__cache and self.__cache[0]:
+            cou=kcwcache.cache().set_config(mycache).get_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])))
+            if cou:
+                self.__None()
+                return cou
+        self.__execute()
+        result = self.__cursor.fetchall() #获取查询结果
+        self.__cursor.close()
+        # self.__None()
+        strs=''
+        if result:
+            strs=result[0][0]
+        if strs and self.__cache and self.__cache[0]:
+            kcwcache.cache().set_config(mycache).set_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])),strs,self.__cache[1])
+        self.__None()
+        return strs
     def count(self,field="*"):
         """查询数量
         
@@ -554,6 +682,11 @@ class mysql:
             self.__sqls="("+self.__sql+")"
             self.__None()
             return self.__sql
+        if self.__cache and self.__cache[0]:
+            cou=kcwcache.cache().set_config(mycache).get_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])))
+            if cou:
+                self.__None()
+                return cou
         self.__execute()
         result = self.__cursor.fetchall() #获取查询结果
         self.__cursor.close()
@@ -565,6 +698,9 @@ class mysql:
             except IndexError:
                 cou=0
         # self.__None()
+        if cou and self.__cache and self.__cache[0]:
+            kcwcache.cache().set_config(mycache).set_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])),cou,self.__cache[1])
+        self.__None()
         return cou
     def max(self,field):
         """查询某字段的最大值
@@ -577,10 +713,18 @@ class mysql:
             self.__sqls="("+self.__sql+")"
             self.__None()
             return self.__sql
+        if self.__cache and self.__cache[0]:
+            cou=kcwcache.cache().set_config(mycache).get_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])))
+            if cou:
+                self.__None()
+                return cou
         self.__execute()
         result = self.__cursor.fetchall() #获取查询结果
         self.__cursor.close()
         cou=int(result[0][0])
+        # self.__None()
+        if cou and self.__cache and self.__cache[0]:
+            kcwcache.cache().set_config(mycache).set_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])),cou,self.__cache[1])
         self.__None()
         return cou
     def min(self,field):
@@ -594,11 +738,19 @@ class mysql:
             self.__sqls="("+self.__sql+")"
             self.__None()
             return self.__sql
+        if self.__cache and self.__cache[0]:
+            cou=kcwcache.cache().set_config(mycache).get_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])))
+            if cou:
+                self.__None()
+                return cou
         self.__execute()
         result = self.__cursor.fetchall() #获取查询结果
         self.__cursor.close()
-        self.__None()
+        # self.__None()
         cou=int(result[0][0])
+        if cou and self.__cache and self.__cache[0]:
+            kcwcache.cache().set_config(mycache).set_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])),cou,self.__cache[1])
+        self.__None()
         return cou
     def avg(self,field):
         """查询某字段的平均值
@@ -611,11 +763,19 @@ class mysql:
             self.__sqls="("+self.__sql+")"
             self.__None()
             return self.__sql
+        if self.__cache and self.__cache[0]:
+            cou=kcwcache.cache().set_config(mycache).get_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])))
+            if cou:
+                self.__None()
+                return cou
         self.__execute()
         result = self.__cursor.fetchall() #获取查询结果
         self.__cursor.close()
-        self.__None()
+        # self.__None()
         cou=int(result[0][0])
+        if cou and self.__cache and self.__cache[0]:
+            kcwcache.cache().set_config(mycache).set_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])),cou,self.__cache[1])
+        self.__None()
         return cou
     def sum(self,field):
         """查询某字段之和
@@ -628,11 +788,22 @@ class mysql:
             self.__sqls="("+self.__sql+")"
             self.__None()
             return self.__sql
+        if self.__cache and self.__cache[0]:
+            cou=kcwcache.cache().set_config(mycache).get_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])))
+            if cou:
+                self.__None()
+                return cou
         self.__execute()
         result = self.__cursor.fetchall() #获取查询结果
         self.__cursor.close()
+        # self.__None()
+        cou=result[0][0]
+        if not cou:
+            cou=0
+        else:
+            if self.__cache and self.__cache[0]:
+                kcwcache.cache().set_config(mycache).set_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])),cou,self.__cache[1])
         self.__None()
-        cou=int(result[0][0])
         return cou
 
     def setinc(self,field,key=1,affair=False):
@@ -762,6 +933,7 @@ class mysql:
         self.__table=None
         self.__buildSql=None
         self.__table=None
+        self.__cache=None
     
     __where=None
     __wheres=()
@@ -919,16 +1091,15 @@ class mysql:
         self.__lock=strs
         return self
    
-    # __cache=[]
-    # def cache(self,endtime,tag=None):
-    #     """设置查询缓存
+    __cache=[]
+    def cache(self,status=None,endtime=86400):
+        """设置查询缓存
+        status  缓存开关
 
-    #     参数 endtime：int 缓存数据  0永久
-
-    #     参数 tag：int 缓存标签
-    #     """
-    #     self.__cache=[endtime,tag]
-    #     return self
+        参数 endtime：int 缓存时间  0永久
+        """
+        self.__cache=[status,endtime]
+        return self
     def __setsql(self,types=None,data = {}):
         """生成sql语句"""
         if types==None:
