@@ -1,7 +1,7 @@
 from queue import Queue
 from .db import model
 from .db import sqlite as kcwsqlite
-import threading,time,os,hashlib,random,traceback
+import threading,time,os,hashlib,random,traceback,multiprocessing
 queuesdbpath=os.path.split(os.path.realpath(__file__))[0]+"/Queues"
 class model_kcweb_task(model.model):
     "任务"
@@ -25,8 +25,23 @@ class model_kcweb_task(model.model):
     }
 class Queues():
     __globalqueue=None
+    # def start():
+    #     if not Queues.__globalqueue:
+    #         if not os.path.isfile(queuesdbpath):
+    #             model_kcweb_tasks=model_kcweb_task()
+    #             model_kcweb_tasks.create_table()
+    #         try:
+    #             kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").find()
+    #         except:
+    #             model_kcweb_tasks=model_kcweb_task()
+    #             model_kcweb_tasks.create_table()
+    #         Queues.__globalqueue=Queue()
+    #         t=threading.Thread(target=Queues.__messagequeue)
+    #         t.daemon=True
+    #         t.start()
     def start():
         if not Queues.__globalqueue:
+            # print("启动")
             if not os.path.isfile(queuesdbpath):
                 model_kcweb_tasks=model_kcweb_task()
                 model_kcweb_tasks.create_table()
@@ -35,35 +50,57 @@ class Queues():
             except:
                 model_kcweb_tasks=model_kcweb_task()
                 model_kcweb_tasks.create_table()
-            Queues.__globalqueue=Queue()
-            t=threading.Thread(target=Queues.__messagequeue)
-            t.daemon=True
+            Queues.__globalqueue=multiprocessing.Queue()
+            t=multiprocessing.Process(target=Queues.messagequeue,args=(Queues.__globalqueue,))
             t.start()
-    
-    def __messagequeue():
-        # time.sleep(0.01)
+    def messagequeue(globalqueue):
+        # print("执行")
         kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").where("code=2 and updtime<"+str(int(time.time())-86400*7)).delete()
         kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").where("code=3 and updtime<"+str(int(time.time())-1200)).delete()
         while True:
-            if not Queues.__globalqueue.empty():
-                value=Queues.__globalqueue.get()
-                kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":3,"msg":"正在执行","error":""})
+            if not globalqueue.empty():
+                value=globalqueue.get()
+                kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":3,"error":""})
                 if value['args']:
                     try:
                         value['target'](*value['args'])
-                    except Exception as e:
-                        kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":1,'endtime':int(time.time()),'updtime':int(time.time()),"msg":"失败:"+str(e),"error":str(traceback.format_exc())})
+                    except:
+                        kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":1,'endtime':int(time.time()),'updtime':int(time.time()),"error":str(traceback.format_exc())})
                     else:
-                        kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":4,"start":100,'endtime':int(time.time()),'updtime':int(time.time()),"msg":"执行完成"})
+                        kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":4,"start":100,'endtime':int(time.time()),'updtime':int(time.time())})
                 else:
                     try:
                         value['target']()
-                    except Exception as e:
-                        kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":1,'endtime':int(time.time()),'updtime':int(time.time()),"msg":"失败:"+str(e),"error":str(traceback.format_exc())})
+                    except:
+                        kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":1,'endtime':int(time.time()),'updtime':int(time.time()),"error":str(traceback.format_exc())})
                     else:
-                        kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":4,"start":100,'endtime':int(time.time()),'updtime':int(time.time()),"msg":"执行完成"})
+                        kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":4,"start":100,'endtime':int(time.time()),'updtime':int(time.time())})
             else:
                 time.sleep(1)
+    # def __messagequeue():
+    #     # time.sleep(0.01)
+    #     kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").where("code=2 and updtime<"+str(int(time.time())-86400*7)).delete()
+    #     kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").where("code=3 and updtime<"+str(int(time.time())-1200)).delete()
+    #     while True:
+    #         if not Queues.__globalqueue.empty():
+    #             value=Queues.__globalqueue.get()
+    #             kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":3,"error":""})
+    #             if value['args']:
+    #                 try:
+    #                     value['target'](*value['args'])
+    #                 except:
+    #                     kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":1,'endtime':int(time.time()),'updtime':int(time.time()),"error":str(traceback.format_exc())})
+    #                 else:
+    #                     kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":4,"start":100,'endtime':int(time.time()),'updtime':int(time.time())})
+    #             else:
+    #                 try:
+    #                     value['target']()
+    #                 except:
+    #                     kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":1,'endtime':int(time.time()),'updtime':int(time.time()),"error":str(traceback.format_exc())})
+    #                 else:
+    #                     kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":4,"start":100,'endtime':int(time.time()),'updtime':int(time.time())})
+    #         else:
+    #             time.sleep(1)
     def delhist():
         """清除任务历史记录(包括 成功的 失败的 已完成的)"""
         return kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").where("code","in",0,1,4).delete()
@@ -100,7 +137,7 @@ class Queues():
                 return False
         else:
             False
-    def insert(target,args=None,title="默认任务",describes="",msg='等待中',taskid=None,start=0,updtime=0): #add_queue
+    def insert(target,args=None,title="默认任务",describes='',msg='',taskid=None,start=0,updtime=0): #add_queue
         """添加队列
         
         target 方法名  必须
